#pragma once

#include <JUI/Widgets/Rect.hpp>
#include <JUI/Widgets/TextRect.hpp>
#include <JUI/Widgets/ListLayout.hpp>


namespace JUI {

    /// A JUI Widget that displays a HSV color input dialog.
    class ColorPicker : public Rect {
    public:
        Event<Color4> OnColorValueChanged;
        void SetColorValue(const Color4& color);
        Color4 GetColorValue() const;

        JUI::TextRect* hex_label;


        float hue = 0;
        float sat = 1.f;
        float bri = 1.f; // AKA val

        ColorPicker() : Rect() {

            Size({100_percent, 100_percent});
            auto* row_layout = new JUI::VerticalListLayout(this);
            row_layout->Padding(5_px);

            hue_slider = new JUI::Slider(row_layout);
            hue_slider->Size({100_percent, 28_px});
            hue_slider->Minimum(0.f); hue_slider->Maximum(1.f);
            hue_slider->Interval(1e-3f);
            hue_slider->ValueChanged += [this] (float val) mutable {
                hue_label->SetContent(std::format("Hue: {}", Math::FloorInt(val*360)));

                hue = val * 360;

                hue_slider->ScrubberColor(Color4::FromHSV(hue, 1.f, 1.f));

                OnColorValueChanged.Invoke(Color4::FromHSV(hue, sat, bri));
            };

            hue_label = new JUI::Text(hue_slider);
            hue_label->AlignCenterHorizontally();
            hue_label->AlignTop();
            hue_label->SetTextColor(Colors::Black);
            hue_label->SetContent("Hue: 0");

            sat_slider = new JUI::Slider(row_layout);
            sat_slider->Size({100_percent, 28_px});
            sat_slider->Minimum(0); sat_slider->Maximum(1);
            sat_slider->Interval(1e-3f);
            sat_slider->ValueChanged += [this] (float val) mutable {
                sat_label->SetContent(std::format("Saturation: {}%", val*100.f));

                sat = val;
                OnColorValueChanged.Invoke(Color4::FromHSV(hue, sat, bri));
            };

            sat_label = new JUI::Text(sat_slider);
            sat_label->AlignCenterHorizontally();
            sat_label->AlignTop();
            sat_label->SetTextColor(Colors::Black);
            sat_label->SetContent("Saturation: 100%");

            bri_slider = new JUI::Slider(row_layout);
            bri_slider->Size({100_percent, 28_px});
            bri_slider->Minimum(0); bri_slider->Maximum(1);
            bri_slider->Interval(1e-3f);
            bri_slider->ValueChanged += [this] (float val) mutable {
                bri_label->SetContent(std::format("Brightness: {}%", val*100.f));

                bri = val;

                OnColorValueChanged.Invoke(Color4::FromHSV(hue, sat, bri));
            };

            bri_label = new JUI::Text(bri_slider);
            bri_label->AlignCenterHorizontally();
            bri_label->AlignTop();
            bri_label->SetTextColor(Colors::Black);
            bri_label->SetContent("Brightness: 100%");



            auto* hue_box = new JUI::Rect();
        }

        explicit ColorPicker(Widget* parent) : ColorPicker() {
            Parent(parent);
        }

    protected:
        JUI::Slider* hue_slider;
        JUI::Slider* sat_slider;
        JUI::Slider* bri_slider;
        JUI::Text* hue_label;
        JUI::Text* sat_label;
        JUI::Text* bri_label;
    private:
    };

}