#include <JUI/Base/TextBase.hpp>
#include <JGL/JGL.h>
using namespace JUI;

void TextBase::SetContent(const std::string& content) { this->content = content; state_redraw = true; }

void TextBase::SetTextColor(const Color4& color) { this->text_color = color; state_redraw = true; }

void TextBase::SetOutlineColor(const Color4& color) { this->outline_color = color; }

void TextBase::SetHorizontalTextAlign(const TextAlign::H& align) { this->h_align = align;}

void TextBase::SetVerticalTextAlign(const TextAlign::V& align) { this->v_align = align; }

void TextBase::SetTextAlign(const TextAlign::H& h_align, const TextAlign::V& v_align) {
    SetHorizontalTextAlign(h_align);
    SetVerticalTextAlign(v_align);
}

void TextBase::AlignLeft() { SetHorizontalTextAlign(TextAlign::H::Left); }

void TextBase::AlignRight() { SetHorizontalTextAlign(TextAlign::H::Right); }

void TextBase::AlignTop() { SetVerticalTextAlign(TextAlign::V::Top); }

void TextBase::AlignBottom() { SetVerticalTextAlign(TextAlign::V::Bottom); }

void TextBase::AlignCenterBoth() {
    SetVerticalTextAlign(TextAlign::V::Center);
    SetHorizontalTextAlign(TextAlign::H::Center);
}

void TextBase::Center() { AlignCenterBoth(); }

void TextBase::AlignCenterHorizontally() {
    SetHorizontalTextAlign(TextAlign::H::Center);
}

void TextBase::AlignCenterVertically() {
    SetVerticalTextAlign(TextAlign::V::Center);
}

void TextBase::SetWordWrap(bool wrap) { word_wrap = wrap; }

void TextBase::Draw(const Vector2& abs_pos, const Vector2& abs_size, const std::string& content, unsigned int size, const Color4& color) {
    // Calculate how much to origin the text based on alignment.
    float align_x = abs_pos.x;
    float align_y = abs_pos.y;

    std::string start = content;
    std::vector<std::string> lines;


    /*for (int i = 0; i < start.length(); i++) {
        std::string substr = start.substr(0, i);
        auto sub_bounds = this->GetFont().MeasureString(substr, size);

        if (sub_bounds.x > abs_size.x)



    }*/


    auto bounds = this->GetFont().MeasureString(content, size);
    // Do nothing if there is no text.
    if (bounds.x == 0 || bounds.y == 0)
        return;

    // Render from left-side of boundary.
    if (h_align == TextAlign::H::Left)
        align_x = abs_pos.x;
    // Render from horizontal center, origin by half of the text's width.
    if (h_align == TextAlign::H::Center)
        align_x = abs_pos.x + (abs_size.x / 2) - (bounds.x / 2);
    // Render from right-side of boundary, origin by text width.
    if (h_align == TextAlign::H::Right)
        align_x = abs_pos.x + abs_size.x - bounds.x;
    // Render from top of boundary.
    if (v_align == TextAlign::V::Top)
        align_y = abs_pos.y;
    // Render from vertical center, origin by half of the text's height.
    if (v_align == TextAlign::V::Center)
        align_y = abs_pos.y + (abs_size.y / 2) - (bounds.y / 2);
    // Render from bottom of boundary, origin by the text's height.
    if (v_align == TextAlign::V::Bottom)
        align_y = abs_pos.y + abs_size.y - bounds.y;

    float scale = 1.f;
    bool use_render_target = false;

    // Has to be checked because this function lets you pass in parameters instead
    // of using the ones we have saved. Which again, defeats the point. - Redacted.
    if (content == this->content) {
        // if Render Target needs updating.
        if (state_redraw) {
            text_canvas->Resize({(int) bounds.x, (int) bounds.y});
            J2D::Begin(text_canvas, true);
                J2D::DrawString(color, content, 0, 0, scale, size, this->set_font);
            J2D::End();
            state_redraw = false;
        }
        use_render_target = true;
    }

    Vector2 text_pos = {align_x, align_y};
    //J2D::Begin();
    //J2D::OutlineRect(color, text_pos, bounds); // Draw bounding box for debugging.
    use_render_target ? J2D::DrawRenderTarget(text_canvas, {align_x, align_y})
    : J2D::DrawString(color, content, align_x, align_y, scale, size, this->set_font);
    //J2D::End();
}

void TextBase::Draw(const Vector2& abs_pos, const Vector2& abs_size) {
    Draw(abs_pos, abs_size, this->content, this->text_size, this->text_color);
}

std::string TextBase::GetContent() const { return content; }

Color4 TextBase::GetTextColor() const { return text_color; }

Color4 TextBase::GetOutlineColor() const { return outline_color; }

TextAlign::H TextBase::GetHorizontalTextAlign() const { return h_align; }

TextAlign::V TextBase::GetVerticalTextAlign() const { return v_align; }

Vector2 TextBase::GetTextBounds() { return GetFont().MeasureString(content, text_size); }

JGL::Font TextBase::GetFont() const { return set_font; }

u32 TextBase::GetTextSize() const { return text_size; }

void TextBase::SetFont(const JGL::Font& font) {
    set_font = font;
    state_redraw = true;
}

void TextBase::SetTextSize(u32 size) {
    text_size = size;
    state_redraw = true;
}

void TextBase::Update(float delta) {}