#include <JUI/Widgets/ContextMenu.hpp>

namespace JUI {
    ContextMenu::ContextMenu(): Rect() {
        Name("ContextMenu");
        this->BGColor(Colors::White);
        //this->Margin(2_px);
        this->Size({200, 200, 0, 0});
        layout = new VerticalListLayout(this);

        MouseExit += [this] (auto _) {
            if (!Pinned())
                Visible(false);
        };
    }

    ContextMenu::ContextMenu(Widget *parent): ContextMenu() {
        this->Parent(parent);
    }

    void ContextMenu::SetFont(const JGL::Font &use_my_font) {
        font = use_my_font;
    }

    float GuesstimateTextWidth(const std::string& text, int ptSize) {
        int length = text.length();
        return length * (ptSize/2.f);
    }

    TextButton * ContextMenu::AddButton(const std::string &name) {
        auto* line_item = new TextButton(layout);
        line_item->SetFont(font);
        line_item->SetContent(name);
        line_item->SetTextSize(16);
        line_item->SetTextColor(Colors::Black);
        line_item->Size({0, 20, 1, 0});

        float line_width = GuesstimateTextWidth(line_item->GetContent(), line_item->GetTextSize());

        if (line_width > estimated_width)
            estimated_width = line_width + 10;

        // TODO: Temp Hack: Implement Auto-size.
        aggregate_height += 20;
        this->size.X.Pixels = estimated_width;
        this->size.Y.Pixels = aggregate_height;

        return line_item;
    }

    TextButton * ContextMenu::AddButton(const std::string &name, const std::function<void()> &callback) {

        auto* btn = AddButton(name);

        // TODO: Is this memory safe at all?
        btn->OnClickEvent += [this, btn, callback]  (auto a, auto b) mutable {
            if (this->IsVisible() && btn->IsMouseInside())
                callback();
        };
        return btn;
    }

    Separator * ContextMenu::AddSeparator(const UDim& size) {
        auto* separator = new Separator(layout);
        separator->Size({100_percent, size});
        return separator;
    }

    ContextMenu * ContextMenu::AddSubmenu(const std::string &name) {
        auto* btn = AddButton(name);

        // TODO: For auto-hiding, check if the widget is over a submenu as well.
        // TODO: Duplicated code with UtilityBar.cpp
        auto* submenu = new JUI::ContextMenu(btn);
        submenu->AnchorPoint({0, 0});
        submenu->Position({100_percent, 0_percent});
        submenu->Visible(false);

        btn->OnClickEvent += [this, submenu] (auto a, auto b) mutable {
            if (this->IsVisible()) {
                this->Pin();
                submenu->Visible(true);
            }

        };

        submenu->MouseExit += [this, submenu] (auto _) {
                if (!submenu->Pinned())
                    this->Unpin();
        };

        return submenu;
    }

    void ContextMenu::CloseOnHoverLoss(bool value) { close_on_hover_loss = value;}

    bool ContextMenu::CloseOnHoverLoss() const { return close_on_hover_loss; }
}
