#include <JUI/Widgets/Slider.hpp>


namespace JUI
{

    Slider::Slider(JUI::Widget *parent) : Slider()
    {
        this->Parent(parent);
    }

    void Slider::OnClick(const Vector2 &MousePos, const MouseButton &MouseButton) {
        Clickable::OnClick(MousePos, MouseButton);
        dragging = true;
    }

    void Slider::OnRelease(const Vector2 &MousePos, const MouseButton &MouseButton, bool MouseStillOver) {
        Clickable::OnRelease(MousePos, MouseButton, MouseStillOver);
        dragging = false;
    }

    void Slider::OnHover(const Vector2 &MousePos) {
        Hoverable::OnHover(MousePos);
    }

    void Slider::OnExit(const Vector2 &MousePos) {
        Hoverable::OnExit(MousePos);
    }

    void Slider::Update(float delta) {
        bool selected = Rect::IsMouseInside();


        hovered = IsMouseInside();

        Hoverable::Update(last_known_mouse_pos, delta);

        // TODO: This is duplicated between here and Window.cpp
        // Will be abstracted away into Clickable shortly.
        if (IsHovered() && mb_state && !prev_mb_state)
        {
            OnClick(last_known_mouse_pos, mbtn);
        }

        if (IsClicked() && !mb_state)
        {
            OnRelease(last_known_mouse_pos, mbtn, IsHovered());
        }

        prev_mb_state = mb_state;

        if (dragging)
        {
            float range = maximum - minimum;
            Vector2 mouse = last_known_mouse_pos;

            float slider_abs_left = this->GetAbsolutePosition().x;
            float slider_abs_right = (this->GetAbsolutePosition().x + this->GetAbsoluteSize().x) - scrubber_width;
            float slider_total_width = this->GetAbsoluteSize().x - scrubber_width;

            float scrubberP = Math::Clamp(mouse.x, slider_abs_left, slider_abs_right);

            float offset = scrubberP - slider_abs_left;

            float percentage = offset / slider_total_width;

            float translated = Math::Clamp(roundMultiple(percentage * Range(), interval), 0.f, Range());

            float value = translated + minimum;

            if (current != value)
                ValueChanged(value);

            current = value;
            //std::cout << "slider value: " << value << std::endl;
        }
        Rect::Update(delta);
    }

    void Slider::Draw() {
        //scrubber->Draw();
        Rect::Draw();
        Vector2 pos = {
                current * (GetAbsoluteSize().x - scrubber_width) + GetAbsolutePosition().x
                ,GetAbsolutePosition().y
        };
        /// TODO: Implement internal padding on scrubber element?
        //J2D::Begin();
        JGL::J2D::FillRect(scrubber_color, pos, {scrubber_width, GetAbsoluteSize().y});
        //J2D::End();
    }

    float Slider::Minimum() const { return minimum; }

    float Slider::Maximum() const { return maximum; }

    float Slider::Interval() const { return interval; }

    float Slider::CurrentValue() const { return current; }

    Color4 Slider::ScrubberColor() const { return scrubber_color; }

    float Slider::ScrubberWidth() const { return scrubber_width; }

    float Slider::Range() const { return maximum - minimum; }

    void Slider::Minimum(float min) { minimum = min; }

    void Slider::Maximum(float max) { maximum = max; }

    void Slider::Interval(float inter) { interval = inter; }

    void Slider::CurrentValue(float value) { current = value; }

    void Slider::ScrubberColor(const Color4 &color) { scrubber_color = color;}

    void Slider::ScrubberWidth(float width) { scrubber_width = width;}

    Slider::Slider() {
        Name("Slider");
    }

    void Slider::SetDragging(bool value) {
        dragging = value;
    }

    bool Slider::Dragging() const { return dragging; }
}